const BorrowTransaction = require('../models/BorrowTransaction');
const ProjectEquipment = require('../models/ProjectEquipment');
const Equipment = require('../models/Equipment');

/**
 * CREATE: เมื่อพนักงานกดยืมอุปกรณ์
 * - ตรวจสอบว่ามี ProjectEquipment (pe_id) นี้จริงหรือไม่
 * - ลดจำนวนใน project_equipments
 * - สร้างรายการ borrow_transactions
 */
exports.createBorrowTransaction = async (req, res) => {
  try {
    // รับฟิลด์จำเป็นจาก req.body
    // เช่น { pe_id, quantity_borrowed, status, ...}
    const {
      pe_id,
      quantity_borrowed,
      status
    } = req.body;

    // 1) หาแถว project_equipments จาก pe_id
    const projectEquipment = await ProjectEquipment.findByPk(pe_id);
    if (!projectEquipment) {
      return res.status(404).json({ error: 'ProjectEquipment not found' });
    }

    // 2) ตรวจสอบว่าจำนวนใน project_equipments พอไหม
    if (projectEquipment.quantity_in_project < quantity_borrowed) {
      return res.status(400).json({ error: 'Not enough equipment in project' });
    }

    // 3) ลดจำนวน
    const updatedQty = projectEquipment.quantity_in_project - quantity_borrowed;
    await projectEquipment.update({ quantity_in_project: updatedQty });

    // 4) สร้าง BorrowTransaction
    const newBorrowTx = await BorrowTransaction.create({
      pe_id, // เก็บ pe_id ใน borrow_transactions
      quantity_borrowed,
      status: status || 'borrowed'
    });

    return res.status(201).json({
      message: 'BorrowTransaction created successfully',
      borrowTransaction: newBorrowTx
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: error.message });
  }
};


/**
 * UPDATE: เมื่อมีการเปลี่ยนสถานะ (เช่น คืนอุปกรณ์)
 * - ถ้า status เปลี่ยนเป็น "returned" -> ถ้าเป็นอุปกรณ์ is_returnable => บวกจำนวนคืน
 */
exports.updateBorrowTransaction = async (req, res) => {
  try {
    const { id } = req.params;  // borrow_id
    const { status } = req.body; // สมมติว่าเราอัปเดตแค่ status, หรืออาจมี field อื่น

    // 1) ค้นหา borrow_transaction เดิม
    const borrowTx = await BorrowTransaction.findByPk(id);
    if (!borrowTx) {
      return res.status(404).json({ error: 'BorrowTransaction not found' });
    }

    // 2) ถ้าเปลี่ยน status -> "returned"
    if (status === 'returned') {
      // 2.1) หา ProjectEquipment
      const projectEquipment = await ProjectEquipment.findByPk(borrowTx.pe_id);
      if (!projectEquipment) {
        return res.status(404).json({ error: 'ProjectEquipment not found' });
      }

      // 2.2) ตรวจสอบว่าอุปกรณ์ returnable ไหม
      // ต้องใช้ equipment_id จาก projectEquipment มาเชื่อมตาราง equipments
      const equipment = await Equipment.findByPk(projectEquipment.equipment_id);
      if (!equipment) {
        return res.status(404).json({ error: 'Equipment not found' });
      }

      // ถ้าเป็นอุปกรณ์ต้องคืน is_returnable = true -> บวกคืน
      if (equipment.is_returnable) {
        const updatedQty = projectEquipment.quantity_in_project + borrowTx.quantity_borrowed;
        await projectEquipment.update({ quantity_in_project: updatedQty });
      }

      // อัปเดตฟิลด์อื่นๆ (เช่น returned_date)
      await borrowTx.update({
        status: 'returned',
        returned_date: new Date()
      });

      return res.status(200).json({
        message: 'BorrowTransaction returned successfully',
        borrowTransaction: borrowTx
      });
    }

    // 3) กรณีอัปเดต status อื่นๆ (ไม่ใช่ returned)
    // ตัวอย่างเช่น update เป็น 'canceled', 'pending', ...
    // สามารถใส่ logic เพิ่มได้ตามต้องการ
    await borrowTx.update({ status });
    
    return res.status(200).json({
      message: 'BorrowTransaction updated successfully',
      borrowTransaction: borrowTx
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: error.message });
  }
};


/**
 * ตัวอย่างฟังก์ชันอื่นๆ (getAll, getById, delete) ...
 */
exports.getAllBorrowTransactions = async (req, res) => {
  try {
    const list = await BorrowTransaction.findAll();
    res.status(200).json(list);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: error.message });
  }
};

exports.getBorrowTransactionById = async (req, res) => {
  try {
    const { id } = req.params;
    const tx = await BorrowTransaction.findByPk(id);
    if (!tx) {
      return res.status(404).json({ error: 'BorrowTransaction not found' });
    }
    return res.status(200).json(tx);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: error.message });
  }
};

exports.deleteBorrowTransaction = async (req, res) => {
  try {
    const { id } = req.params;
    const rowsDeleted = await BorrowTransaction.destroy({ where: { borrow_id: id } });
    if (!rowsDeleted) {
      return res.status(404).json({ error: 'BorrowTransaction not found' });
    }
    return res.status(200).json({ message: 'BorrowTransaction deleted' });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: error.message });
  }
};
